/*
 * Copyright (c) 2005 Massachusetts Institute of Technology
 *                    All rights reserved
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "includes.h"
RCSID("$OpenBSD: readconf.c,v 1.137 2005/03/04 08:48:06 djm Exp $");

#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include "log.h"
#include "misc.h"
#include "sshhop.h"
#include "xmalloc.h"
#include "ssh.h"
#include "readconf.h"

#undef  _PATH_SSH_PROGRAM
#define _PATH_SSH_PROGRAM	"./ssh"

Options options;
int     debug_flag;

/* !!!NOTE: Toy program -- no signal handlers !!!*/

/* Prints a help message to the user.  This function never returns. */
static void
usage(void)
{
	fprintf(stderr,
"usage: sshhop_test [ssh_options] -H [ssh_options] [-p port1] [user1@]host1\n"
"                             ... -H [ssh_options] [-p portn] [usern@]hostn\n"
"                   [command]\n"
	);
	exit(1);
}

/*
 * Main for sshhop test program.
 */
int
main(int argc, char *argv[])
{
	int hops;
	int error;
	int original_real_uid;
	char *ssh_path = xstrdup(_PATH_SSH_PROGRAM);
	struct passwd *pw;

	/* May be enough arguments */
	if (argc > 4) {
		original_real_uid = getuid();
		/* Get user data. */
		pw = getpwuid(original_real_uid);
		if (!pw) {
			logit("You don't exist, go away!");
			exit(1);
		}
		/* Take a copy of the returned structure. */
		pw = pwcopy(pw);

		hops = sshop_parse_options(argc, argv, usage);
		/* No hosts given, exit after printing usage */
		if (hops <= 0) {
			usage();
		}
		/* Only one host, run the standard SSH.
		 * Overwrite this process */
		else if (hops == 1) {
			argv[0] = ssh_path;
			execv(argv[0], argv);
		}
		/* More than one hop, set up the hop chain */
		else {
			/*
		 	* Initialize "log" output.  Since we are the client all output
		 	* actually goes to stderr.
		 	*/
			log_init(argv[0],
			    options.log_level == -1 ? SYSLOG_LEVEL_INFO : options.log_level,
			    SYSLOG_FACILITY_USER, 1);

			/* Make sure the user is set */
			sshop_set_default_user(pw->pw_name);

			/* Build the hop chain */
			error = sshop_build_chain(ssh_path);

			/* Hop chain established successfully, wait to clean
			 * processes up.
			 */
			if (error == 0) {
				sshop_wait_for_exit();
			/* Things were not started properly, shutdown */
			} else {
				sshop_kill_subprocesses();
				sshop_wait_for_exit();
			}
		}
	/* Not enough arguments */
	} else {
		usage();
	}
	return(0);
}

